#
# $FreeBSD: src/etc/etc.amd64/ttys,v 1.18.2.1 2009/08/03 08:13:06 kensmith Exp $
#	@(#)ttys	5.1 (Berkeley) 4/17/89
#
# This file specifies various information about terminals on the system.
# It is used by several different programs.  Common entries for the
# various columns include:
#
# name  The name of the terminal device.
#
# getty The program to start running on the terminal.  Typically a
#       getty program, as the name implies.  Other common entries
#       include none, when no getty is needed, and xdm, to start the
#       X Window System.
#
# type The initial terminal type for this port.  For hardwired
#      terminal lines, this will contain the type of terminal used.
#      For virtual consoles, the correct type is typically xterm.
#      Other common values include network for network connections on
#      pseudo-terminals, dialup for incoming modem ports, and unknown
#      when the terminal type cannot be predetermined.
#
# status Must be on or off.  If on, init will run the getty program on
#        the specified port.  If the word "secure" appears, this tty
#        allows root login.
#
# name	getty				type	status		comments
#
# If console is marked "insecure", then init will ask for the root password
# when going to single-user mode.
<%
    import os
    import contextlib
    from bsd import sysctl
    from freenas.utils import first_or_default

    adv = dispatcher.call_sync('system.advanced.get_config')
    serial = 'on' if adv.get('serial_console') else 'off'

    with contextlib.suppress(OSError):
        os.unlink('/dev/sercons')

    if serial == 'on':
        configured, _ = sysctl.sysctlbyname('kern.console').split('/')
        sercons = first_or_default(lambda p: p.startswith('ttyu'), configured.split(','))
        with contextlib.suppress(OSError):
            os.symlink(sercons or 'ttyu0', '/dev/sercons')
%>\
console	none				unknown	off secure
#
ttyv0	"/usr/libexec/getty go-cli"	xterm	on  secure
% if adv.get('console_cli'):
ttyv1	"/usr/libexec/getty cli"	xterm	on  secure
% else:
ttyv1	"/usr/libexec/getty Pc"		xterm	on  secure
% endif
# Virtual terminals
ttyv2	"/usr/libexec/getty Pc"		xterm	on  secure
ttyv3	"/usr/libexec/getty Pc"		xterm	on  secure
ttyv4	"/usr/libexec/getty Pc"		xterm	on  secure
ttyv5	"/usr/libexec/getty Pc"		xterm	on  secure
ttyv6	"/usr/libexec/getty Pc"		xterm	on  secure
ttyv7	"/usr/libexec/getty Pc"		xterm	on  secure
ttyv8	"/usr/local/bin/xdm -nodaemon"	xterm	off secure
# Serial terminals
# The 'dialup' keyword identifies dialin lines to login, fingerd etc.
% if adv.get('console_cli'):
sercons	"/usr/libexec/getty cli_serial${adv.get('serial_speed')}"	vt100	${serial} secure
% else:
sercons	"/usr/libexec/getty std.${adv.get('serial_speed')}"	vt100	${serial} secure
% endif
ttyu0	"/usr/libexec/getty std.9600"	dialup	off secure
ttyu1	"/usr/libexec/getty std.9600"	dialup	off secure
ttyu2	"/usr/libexec/getty std.9600"	dialup	off secure
ttyu3	"/usr/libexec/getty std.9600"	dialup	off secure
# Dumb console
dcons	"/usr/libexec/getty std.9600"	vt100	off secure
# Pseudo terminals
ttyp0	none			network
ttyp1	none			network
ttyp2	none			network
ttyp3	none			network
ttyp4	none			network
ttyp5	none			network
ttyp6	none			network
ttyp7	none			network
ttyp8	none			network
ttyp9	none			network
ttypa	none			network
ttypb	none			network
ttypc	none			network
ttypd	none			network
ttype	none			network
ttypf	none			network
ttypg	none			network
ttyph	none			network
ttypi	none			network
ttypj	none			network
ttypk	none			network
ttypl	none			network
ttypm	none			network
ttypn	none			network
ttypo	none			network
ttypp	none			network
ttypq	none			network
ttypr	none			network
ttyps	none			network
ttypt	none			network
ttypu	none			network
ttypv	none			network
ttyq0	none			network
ttyq1	none			network
ttyq2	none			network
ttyq3	none			network
ttyq4	none			network
ttyq5	none			network
ttyq6	none			network
ttyq7	none			network
ttyq8	none			network
ttyq9	none			network
ttyqa	none			network
ttyqb	none			network
ttyqc	none			network
ttyqd	none			network
ttyqe	none			network
ttyqf	none			network
ttyqg	none			network
ttyqh	none			network
ttyqi	none			network
ttyqj	none			network
ttyqk	none			network
ttyql	none			network
ttyqm	none			network
ttyqn	none			network
ttyqo	none			network
ttyqp	none			network
ttyqq	none			network
ttyqr	none			network
ttyqs	none			network
ttyqt	none			network
ttyqu	none			network
ttyqv	none			network
ttyr0	none			network
ttyr1	none			network
ttyr2	none			network
ttyr3	none			network
ttyr4	none			network
ttyr5	none			network
ttyr6	none			network
ttyr7	none			network
ttyr8	none			network
ttyr9	none			network
ttyra	none			network
ttyrb	none			network
ttyrc	none			network
ttyrd	none			network
ttyre	none			network
ttyrf	none			network
ttyrg	none			network
ttyrh	none			network
ttyri	none			network
ttyrj	none			network
ttyrk	none			network
ttyrl	none			network
ttyrm	none			network
ttyrn	none			network
ttyro	none			network
ttyrp	none			network
ttyrq	none			network
ttyrr	none			network
ttyrs	none			network
ttyrt	none			network
ttyru	none			network
ttyrv	none			network
ttys0	none			network
ttys1	none			network
ttys2	none			network
ttys3	none			network
ttys4	none			network
ttys5	none			network
ttys6	none			network
ttys7	none			network
ttys8	none			network
ttys9	none			network
ttysa	none			network
ttysb	none			network
ttysc	none			network
ttysd	none			network
ttyse	none			network
ttysf	none			network
ttysg	none			network
ttysh	none			network
ttysi	none			network
ttysj	none			network
ttysk	none			network
ttysl	none			network
ttysm	none			network
ttysn	none			network
ttyso	none			network
ttysp	none			network
ttysq	none			network
ttysr	none			network
ttyss	none			network
ttyst	none			network
ttysu	none			network
ttysv	none			network
ttyP0	none			network
ttyP1	none			network
ttyP2	none			network
ttyP3	none			network
ttyP4	none			network
ttyP5	none			network
ttyP6	none			network
ttyP7	none			network
ttyP8	none			network
ttyP9	none			network
ttyPa	none			network
ttyPb	none			network
ttyPc	none			network
ttyPd	none			network
ttyPe	none			network
ttyPf	none			network
ttyPg	none			network
ttyPh	none			network
ttyPi	none			network
ttyPj	none			network
ttyPk	none			network
ttyPl	none			network
ttyPm	none			network
ttyPn	none			network
ttyPo	none			network
ttyPp	none			network
ttyPq	none			network
ttyPr	none			network
ttyPs	none			network
ttyPt	none			network
ttyPu	none			network
ttyPv	none			network
ttyQ0	none			network
ttyQ1	none			network
ttyQ2	none			network
ttyQ3	none			network
ttyQ4	none			network
ttyQ5	none			network
ttyQ6	none			network
ttyQ7	none			network
ttyQ8	none			network
ttyQ9	none			network
ttyQa	none			network
ttyQb	none			network
ttyQc	none			network
ttyQd	none			network
ttyQe	none			network
ttyQf	none			network
ttyQg	none			network
ttyQh	none			network
ttyQi	none			network
ttyQj	none			network
ttyQk	none			network
ttyQl	none			network
ttyQm	none			network
ttyQn	none			network
ttyQo	none			network
ttyQp	none			network
ttyQq	none			network
ttyQr	none			network
ttyQs	none			network
ttyQt	none			network
ttyQu	none			network
ttyQv	none			network
ttyR0	none			network
ttyR1	none			network
ttyR2	none			network
ttyR3	none			network
ttyR4	none			network
ttyR5	none			network
ttyR6	none			network
ttyR7	none			network
ttyR8	none			network
ttyR9	none			network
ttyRa	none			network
ttyRb	none			network
ttyRc	none			network
ttyRd	none			network
ttyRe	none			network
ttyRf	none			network
ttyRg	none			network
ttyRh	none			network
ttyRi	none			network
ttyRj	none			network
ttyRk	none			network
ttyRl	none			network
ttyRm	none			network
ttyRn	none			network
ttyRo	none			network
ttyRp	none			network
ttyRq	none			network
ttyRr	none			network
ttyRs	none			network
ttyRt	none			network
ttyRu	none			network
ttyRv	none			network
ttyS0	none			network
ttyS1	none			network
ttyS2	none			network
ttyS3	none			network
ttyS4	none			network
ttyS5	none			network
ttyS6	none			network
ttyS7	none			network
ttyS8	none			network
ttyS9	none			network
ttySa	none			network
ttySb	none			network
ttySc	none			network
ttySd	none			network
ttySe	none			network
ttySf	none			network
ttySg	none			network
ttySh	none			network
ttySi	none			network
ttySj	none			network
ttySk	none			network
ttySl	none			network
ttySm	none			network
ttySn	none			network
ttySo	none			network
ttySp	none			network
ttySq	none			network
ttySr	none			network
ttySs	none			network
ttySt	none			network
ttySu	none			network
ttySv	none			network
