'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.DispatcherClient = exports.RPCException = exports.EntitySubscriber = exports.FileClient = exports.ContainerConsoleClient = exports.ShellClient = exports.StreamClient = exports.getCode = exports.getErrno = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _ShellClient = require('./ShellClient');

Object.defineProperty(exports, 'StreamClient', {
    enumerable: true,
    get: function get() {
        return _ShellClient.StreamClient;
    }
});
Object.defineProperty(exports, 'ShellClient', {
    enumerable: true,
    get: function get() {
        return _ShellClient.ShellClient;
    }
});
Object.defineProperty(exports, 'ContainerConsoleClient', {
    enumerable: true,
    get: function get() {
        return _ShellClient.ContainerConsoleClient;
    }
});

var _FileClient = require('./FileClient');

Object.defineProperty(exports, 'FileClient', {
    enumerable: true,
    get: function get() {
        return _FileClient.FileClient;
    }
});

var _EntitySubscriber = require('./EntitySubscriber.js');

Object.defineProperty(exports, 'EntitySubscriber', {
    enumerable: true,
    get: function get() {
        return _EntitySubscriber.EntitySubscriber;
    }
});

var _ErrnoCodes = require('./ErrnoCodes.js');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 * Copyright 2015 iXsystems, Inc.
 * All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted providing that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
require("babel-polyfill");

var INVALID_JSON_RESPONSE = 1;
var CONNECTION_TIMEOUT = 2;
var CONNECTION_CLOSED = 3;
var RPC_CALL_TIMEOUT = 4;
var RPC_CALL_ERROR = 5;
var SPURIOUS_RPC_RESPONSE = 6;
var LOGOUT = 7;
var OTHER = 8;

exports.getErrno = _ErrnoCodes.getErrno;
exports.getCode = _ErrnoCodes.getCode;

var RPCException = exports.RPCException = function RPCException(code, message) {
    var extra = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
    var stacktrace = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

    _classCallCheck(this, RPCException);

    this.code = code;
    this.message = message;
    this.extra = extra;
    this.stacktrace = stacktrace;
};

var DispatcherClient = exports.DispatcherClient = function () {
    function DispatcherClient(hostname) {
        _classCallCheck(this, DispatcherClient);

        this.defaultTimeout = 20;
        this.hostname = hostname;
        this.socket = null;
        this.reconnectInterval = null;
        this.reconnectTimer = null;
        this.keepaliveInterval = 60;
        this.keepaliveTimer = null;
        this.pendingCalls = new Map();
        this.eventHandlers = new Map();
        this.subscriptions = new Map();

        /* Callbacks */
        this.onEvent = function () {};
        this.onConnect = function () {};
        this.onDisconnect = function () {};
        this.onLogin = function () {};
        this.onRPCResponse = function () {};
        this.onError = function () {};
    }

    _createClass(DispatcherClient, [{
        key: '__onmessage',
        value: function __onmessage(msg) {
            try {
                var data = JSON.parse(msg.data);
            } catch (e) {
                console.warn('Malformed response: "' + msg.data + '"');
                this.onError(INVALID_JSON_RESPONSE);
                return;
            }

            if (data.namespace == "events" && data.name == "event") {
                if (this.eventHandlers.has(data.args.name)) {
                    var _iteratorNormalCompletion = true;
                    var _didIteratorError = false;
                    var _iteratorError = undefined;

                    try {
                        for (var _iterator = this.eventHandlers.get(data.args.name).values()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                            var e = _step.value;

                            e(data.args.args);
                        }
                    } catch (err) {
                        _didIteratorError = true;
                        _iteratorError = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion && _iterator.return) {
                                _iterator.return();
                            }
                        } finally {
                            if (_didIteratorError) {
                                throw _iteratorError;
                            }
                        }
                    }
                }

                this.onEvent(data.args.name, data.args.args);
                return;
            }

            if (data.namespace == "events" && data.name == "logout") {
                this.onError(LOGOUT);
                return;
            }

            if (data.namespace == "rpc") {
                if (data.name == "call") {
                    console.error("Server functionality is not supported");
                    this.onError(SPURIOUS_RPC_RESPONSE);
                    return;
                }

                if (data.name == "response" || data.name == "error") {
                    if (!this.pendingCalls.has(data.id)) {
                        console.warn('Spurious RPC response: ' + data.id);
                        this.onError(SPURIOUS_RPC_RESPONSE);
                        return;
                    }

                    var result = data.args;
                    if (data.name == "error") {
                        result = new RPCException(data.args.code, data.args.message, data.args.extra, data.args.stacktrace);
                    }

                    var call = this.pendingCalls.get(data.id);
                    clearTimeout(call.timeout);

                    /* Handle the promise */
                    if (result instanceof RPCException) call.reject !== undefined && call.reject(result);else call.resolve !== undefined && call.resolve(result);

                    if (call.callback !== null) call.callback(result);

                    this.pendingCalls.delete(data.id);
                }
            }
        }
    }, {
        key: '__onopen',
        value: function __onopen() {
            console.log("Connection established");
            this.onConnect();
        }
    }, {
        key: '__onclose',
        value: function __onclose() {
            var errno = (0, _ErrnoCodes.getCode)("ECONNRESET");

            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = this.pendingCalls.values()[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var call = _step2.value;

                    call.callback(new RPCException(errno.code, errno.description));
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }

            console.log("Connection closed");
            this.onDisconnect();
        }
    }, {
        key: '__ontimeout',
        value: function __ontimeout(id) {
            var call = this.pendingCalls.get(id);
            var errno = (0, _ErrnoCodes.getCode)("ETIMEDOUT");

            call.callback(new RPCException(errno.code, errno.description));

            this.pendingCalls.delete(id);
            this.onError(RPC_CALL_TIMEOUT, call.method, call.args);
        }
    }, {
        key: 'connect',
        value: function connect() {
            this.socket = new WebSocket('ws://' + this.hostname + '/dispatcher/socket');
            this.socket.onmessage = this.__onmessage.bind(this);
            this.socket.onopen = this.__onopen.bind(this);
            this.socket.onclose = this.__onclose.bind(this);

            if (this.reconnectTimer) {
                clearTimeout(this.reconnectTimer);
                this.reconnectTimer = null;
            }
        }
    }, {
        key: 'disconnect',
        value: function disconnect() {
            var _this = this;

            if (this.reconnectInterval) {
                this.reconnectTimer = setTimeout(function () {
                    _this.connect();
                    _this.login_token(_this.token);
                }, this.reconnectInterval);
            }

            if (this.keepaliveTimer) {
                clearTimeout(this.keepaliveTimer);
                this.keepaliveTimer = null;
            }

            this.socket.close();
        }
    }, {
        key: 'ping',
        value: function ping() {
            this.call("management.ping", [], function () {});
        }
    }, {
        key: 'login',
        value: function login(username, password) {
            var _this2 = this;

            var id = DispatcherClient.__uuid();
            var payload = {
                "username": username,
                "password": password
            };

            this.pendingCalls.set(id, {
                "callback": function callback(result) {
                    if (_this2.keepaliveInterval) {
                        _this2.keepaliveTimer = setTimeout(_this2.ping.bind(_this2), _this2.keepaliveInterval);
                    }

                    _this2.token = result;
                    _this2.onLogin(result);
                }
            });

            this.socket.send(DispatcherClient.__pack("rpc", "auth", payload, id));
        }
    }, {
        key: 'login_token',
        value: function login_token(token) {
            var _this3 = this;

            var id = DispatcherClient.__uuid();
            var payload = { "token": token };

            this.pendingCalls.set(id, {
                "callback": function callback(result) {
                    return _this3.onLogin(result);
                }
            });

            this.socket.send(DispatcherClient.__pack("rpc", "auth_token", payload, id));
        }
    }, {
        key: 'call',
        value: function call(method, args) {
            var _this4 = this;

            var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

            return new Promise(function (resolve, reject) {
                var id = DispatcherClient.__uuid();
                var timeout = setTimeout(function () {
                    _this4.__ontimeout(id);
                }, _this4.defaultTimeout * 1000);
                var payload = {
                    "method": method,
                    "args": args
                };

                _this4.pendingCalls.set(id, {
                    "method": method,
                    "args": args,
                    "callback": callback,
                    "resolve": resolve,
                    "reject": reject,
                    "timeout": timeout
                });

                _this4.socket.send(DispatcherClient.__pack("rpc", "call", payload, id));
            });
        }
    }, {
        key: 'subscribe',
        value: function subscribe(pattern) {
            if (!this.subscriptions.has(pattern)) {
                this.socket.send(DispatcherClient.__pack("events", "subscribe", [pattern]));
                this.subscriptions.set(pattern, 0);
            }

            var refcount = this.subscriptions.get(pattern) + 1;
            this.subscriptions.set(pattern, refcount);
        }
    }, {
        key: 'unsubscribe',
        value: function unsubscribe(pattern) {
            if (!this.subscriptions.has(pattern)) {
                throw new Error('No previous subscription for ' + pattern);
            }

            var refcount = this.subscriptions.get(pattern) - 1;
            this.subscriptions.set(pattern, refcount);

            if (refcount === 0) {
                this.socket.send(DispatcherClient.__pack("events", "unsubscribe", [pattern]));
                this.subscriptions.delete(pattern);
            }
        }
    }, {
        key: 'emitEvent',
        value: function emitEvent(name, args) {
            this.socket.send(DispatcherClient.__pack("events", "event", {
                "name": name,
                "args": "args"
            }));
        }
    }, {
        key: 'registerEventHandler',
        value: function registerEventHandler(name, callback) {
            if (!this.eventHandlers.has(name)) {
                this.eventHandlers.set(name, new Map());
            }

            var cookie = DispatcherClient.__uuid();
            var list = this.eventHandlers.get(name);
            list.set(cookie, callback);
            this.subscribe(name);
        }
    }, {
        key: 'unregisterEventHandler',
        value: function unregisterEventHandler(name, cookie) {
            if (!this.eventHandlers.has(name)) {
                throw new Error('there are no handlers registered for ' + name);
            }

            var list = this.eventHandlers.get(name);
            if (!list.has(cookie)) {
                throw new Error("no handler registered for that cookie");
            }

            this.unsubscribe(name);
            list.delete(cookie);
        }
    }], [{
        key: '__uuid',
        value: function __uuid() {
            return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
                var r = Math.random() * 16 | 0,
                    v = c == "x" ? r : r & 0x3 | 0x8;
                return v.toString(16);
            });
        }
    }, {
        key: '__pack',
        value: function __pack(namespace, name, args, id) {
            return JSON.stringify({
                "namespace": namespace,
                "id": id || DispatcherClient.__uuid(),
                "name": name,
                "args": args
            });
        }
    }]);

    return DispatcherClient;
}();