'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 * Copyright 2015 iXsystems, Inc.
 * All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted providing that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

var diff = require('deep-diff').diff;

var operatorsTable = {
    '=': function _(x, y) {
        return x == y;
    },
    '!=': function _(x, y) {
        return x != y;
    },
    '>': function _(x, y) {
        return x > y;
    },
    '<': function _(x, y) {
        return x < y;
    },
    '>=': function _(x, y) {
        return x >= y;
    },
    '<=': function _(x, y) {
        return x <= y;
    },
    '~': function _(x, y) {
        return x.match(y);
    },
    'in': function _in(x, y) {
        return y.indexOf(x) > -1;
    },
    'nin': function nin(x, y) {
        return y.indexOf(x) == -1;
    }
};

var conversions_table = {
    'timestamp': function timestamp(v) {
        return Date.parse(v) / 1000;
    }
};

function evalLogicAnd(item, lst) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = lst[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var _i = _step.value;

            if (!evalTuple(item, _i)) {
                return false;
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return true;
}

function evalLogicOr(item, lst) {
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
        for (var _iterator2 = lst[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var _i2 = _step2.value;

            if (evalTuple(item, _i2)) {
                return true;
            }
        }
    } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
            }
        } finally {
            if (_didIteratorError2) {
                throw _iteratorError2;
            }
        }
    }

    return false;
}

function evalLogicNor(item, lst) {
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
        for (var _iterator3 = lst[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var _i3 = _step3.value;

            if (evalTuple(item, _i3)) {
                return false;
            }
        }
    } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
                _iterator3.return();
            }
        } finally {
            if (_didIteratorError3) {
                throw _iteratorError3;
            }
        }
    }

    return true;
}

function evalLogicOperator(item, t) {
    var _t = _slicedToArray(t, 2),
        op = _t[0],
        lst = _t[1];

    switch (op) {
        case and:
            return evalLogicAnd(item, lst);
        case or:
            return evalLogicOr(item, lst);
        case nor:
            return evalLogicNor(item, lst);
    }
}

function evalFieldOperator(item, t) {
    var _t2 = _slicedToArray(t, 3),
        left = _t2[0],
        op = _t2[1],
        right = _t2[2];

    if (Object.keys(t).length == 4) {
        right = conversions_table[t[3]](right);
    }

    return operatorsTable[op](item[left], right);
}

function evalTuple(item, t) {
    if (Object.keys(t).length == 2) {
        return evalLogicOperator(item, t);
    }
    if (Object.keys(t).length == 3 || Object.keys(t).length == 4) {
        return evalFieldOperator(item, t);
    }
}

function matches(obj, rules) {
    var fail = false;
    var _iteratorNormalCompletion4 = true;
    var _didIteratorError4 = false;
    var _iteratorError4 = undefined;

    try {
        for (var _iterator4 = rules[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
            var r = _step4.value;

            if (!evalTuple(obj, r)) {
                fail = true;
                break;
            }
        }
    } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion4 && _iterator4.return) {
                _iterator4.return();
            }
        } finally {
            if (_didIteratorError4) {
                throw _iteratorError4;
            }
        }
    }

    return !fail;
}

var CappedMap = function () {
    function CappedMap(maxsize) {
        _classCallCheck(this, CappedMap);

        this.map = new Map();
        this.maxsize = maxsize;
    }

    _createClass(CappedMap, [{
        key: 'get',
        value: function get(key) {
            return this.map.get(key);
        }
    }, {
        key: 'set',
        value: function set(key, value) {
            this.map.set(key, value);
            if (this.map.size > this.maxsize) {
                var mapIter = this.map.entries();
                this.map.delete(mapIter.next().key);
            }
        }
    }, {
        key: 'has',
        value: function has(key) {
            return this.map.has(key);
        }
    }, {
        key: 'getSize',
        value: function getSize() {
            return this.map.size;
        }
    }, {
        key: 'setMaxSize',
        value: function setMaxSize(maxsize) {
            if (maxsize < this.maxsize) {
                var mapIter = this.map.entries();
                var deleteSize = this.map.size - maxsize;
                for (i = 0; i < deleteSize; i++) {
                    this.map.delete(mapIter.next().key);
                }
            }
            this.maxsize = maxsize;
        }
    }, {
        key: 'deleteKey',
        value: function deleteKey(key) {
            this.map.delete(key);
        }
    }, {
        key: 'entries',
        value: function entries() {
            return this.map.entries();
        }
    }]);

    return CappedMap;
}();

var EntitySubscriber = exports.EntitySubscriber = function () {
    function EntitySubscriber(client, name) {
        var maxsize = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 2000;

        _classCallCheck(this, EntitySubscriber);

        this.name = name;
        this.client = client;
        this.maxsize = maxsize;
        this.objects = new CappedMap(maxsize);
        this.handlerCookie = null;

        /* Callbacks */
        this.onCreate = function () {};
        this.onUpdate = function () {};
        this.onDelete = function () {};
    }

    _createClass(EntitySubscriber, [{
        key: 'fetch',
        value: function fetch() {
            var _this = this;

            this.client.call(this.name + '.query', [[], { "limit": this.maxsize }], function (result) {
                var _iteratorNormalCompletion5 = true;
                var _didIteratorError5 = false;
                var _iteratorError5 = undefined;

                try {
                    for (var _iterator5 = result[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
                        var _i4 = _step5.value;

                        _this.objects.set(_i4.id, _i4);
                    }
                } catch (err) {
                    _didIteratorError5 = true;
                    _iteratorError5 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion5 && _iterator5.return) {
                            _iterator5.return();
                        }
                    } finally {
                        if (_didIteratorError5) {
                            throw _iteratorError5;
                        }
                    }
                }
            });
        }
    }, {
        key: 'start',
        value: function start() {
            this.handlerCookie = this.client.registerEventHandler('entity-subscriber.' + this.name + '.changed', this.__onChanged.bind(this));
        }
    }, {
        key: 'stop',
        value: function stop() {
            this.client.unregisterEventHandler('entity-subscriber.' + this.name + '.changed', this.handlerCookie);
        }
    }, {
        key: '__onChanged',
        value: function __onChanged(event) {
            if (event.operation == "create") {
                var _iteratorNormalCompletion6 = true;
                var _didIteratorError6 = false;
                var _iteratorError6 = undefined;

                try {
                    for (var _iterator6 = event.entities[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
                        var entity = _step6.value;

                        this.objects.set(entity.id, entity);
                    }
                } catch (err) {
                    _didIteratorError6 = true;
                    _iteratorError6 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion6 && _iterator6.return) {
                            _iterator6.return();
                        }
                    } finally {
                        if (_didIteratorError6) {
                            throw _iteratorError6;
                        }
                    }
                }
            }

            if (event.operation == "update") {
                var _iteratorNormalCompletion7 = true;
                var _didIteratorError7 = false;
                var _iteratorError7 = undefined;

                try {
                    for (var _iterator7 = event.entities[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
                        var _entity = _step7.value;

                        this.objects.set(_entity.id, _entity);
                    }
                } catch (err) {
                    _didIteratorError7 = true;
                    _iteratorError7 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion7 && _iterator7.return) {
                            _iterator7.return();
                        }
                    } finally {
                        if (_didIteratorError7) {
                            throw _iteratorError7;
                        }
                    }
                }
            }

            if (event.operation == "rename") {
                var _iteratorNormalCompletion8 = true;
                var _didIteratorError8 = false;
                var _iteratorError8 = undefined;

                try {
                    for (var _iterator8 = event.ids[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
                        var _step8$value = _slicedToArray(_step8.value, 2),
                            old_id = _step8$value[0],
                            new_id = _step8$value[1];

                        if (this.objects.has(old_id)) {
                            var _entity2 = this.objects.get(old_id);
                            _entity2.id = new_id;
                            this.objects.set(_entity2.id, _entity2);
                            this.objects.deleteKey(old_id);
                        }
                    }
                } catch (err) {
                    _didIteratorError8 = true;
                    _iteratorError8 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion8 && _iterator8.return) {
                            _iterator8.return();
                        }
                    } finally {
                        if (_didIteratorError8) {
                            throw _iteratorError8;
                        }
                    }
                }
            }

            if (event.operation == "delete") {
                var _iteratorNormalCompletion9 = true;
                var _didIteratorError9 = false;
                var _iteratorError9 = undefined;

                try {
                    for (var _iterator9 = event.ids[Symbol.iterator](), _step9; !(_iteratorNormalCompletion9 = (_step9 = _iterator9.next()).done); _iteratorNormalCompletion9 = true) {
                        var id = _step9.value;

                        if (this.objects.has(id)) this.objects.deleteKey(id);
                    }
                } catch (err) {
                    _didIteratorError9 = true;
                    _iteratorError9 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion9 && _iterator9.return) {
                            _iterator9.return();
                        }
                    } finally {
                        if (_didIteratorError9) {
                            throw _iteratorError9;
                        }
                    }
                }
            }
        }
    }, {
        key: 'query',
        value: function query(rules, params, callback) {
            if (this.objects.getSize() == this.maxsize) {
                this.client.call(this.name + '.query', [rules, params], callback);
            } else {
                var _single = params.single;
                var _count = params.count;
                var _offset = params.offset;
                var _limit = params.limit;
                var _sort = params.sort;
                var _result = new Array();

                if (Object.keys(rules).length == 0) {
                    var _iteratorNormalCompletion10 = true;
                    var _didIteratorError10 = false;
                    var _iteratorError10 = undefined;

                    try {
                        for (var _iterator10 = this.objects.entries()[Symbol.iterator](), _step10; !(_iteratorNormalCompletion10 = (_step10 = _iterator10.next()).done); _iteratorNormalCompletion10 = true) {
                            var _step10$value = _slicedToArray(_step10.value, 2),
                                key = _step10$value[0],
                                value = _step10$value[1];

                            _result.push(value);
                        }
                    } catch (err) {
                        _didIteratorError10 = true;
                        _iteratorError10 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion10 && _iterator10.return) {
                                _iterator10.return();
                            }
                        } finally {
                            if (_didIteratorError10) {
                                throw _iteratorError10;
                            }
                        }
                    }
                } else {
                    var _iteratorNormalCompletion11 = true;
                    var _didIteratorError11 = false;
                    var _iteratorError11 = undefined;

                    try {
                        for (var _iterator11 = this.objects.entries()[Symbol.iterator](), _step11; !(_iteratorNormalCompletion11 = (_step11 = _iterator11.next()).done); _iteratorNormalCompletion11 = true) {
                            var _step11$value = _slicedToArray(_step11.value, 2),
                                key = _step11$value[0],
                                value = _step11$value[1];

                            if (matches(value, rules)) {
                                _result.push(value);
                            }
                        }
                    } catch (err) {
                        _didIteratorError11 = true;
                        _iteratorError11 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion11 && _iterator11.return) {
                                _iterator11.return();
                            }
                        } finally {
                            if (_didIteratorError11) {
                                throw _iteratorError11;
                            }
                        }
                    }
                }

                if (_sort) {
                    var sortKeys = [];
                    var direction = [];
                    var _iteratorNormalCompletion12 = true;
                    var _didIteratorError12 = false;
                    var _iteratorError12 = undefined;

                    try {
                        for (var _iterator12 = _sort[Symbol.iterator](), _step12; !(_iteratorNormalCompletion12 = (_step12 = _iterator12.next()).done); _iteratorNormalCompletion12 = true) {
                            var _i5 = _step12.value;

                            if (_i5.charAt(0) == '-') {
                                sortKeys.push(_i5.slice(1));
                                direction.push('desc');
                            } else {
                                sortKeys.push(_i5);
                                direction.push('asc');
                            }
                        }
                    } catch (err) {
                        _didIteratorError12 = true;
                        _iteratorError12 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion12 && _iterator12.return) {
                                _iterator12.return();
                            }
                        } finally {
                            if (_didIteratorError12) {
                                throw _iteratorError12;
                            }
                        }
                    }

                    _.map(_.sortByOrder(_result, sortKeys, direction), _.values);
                }

                if (_offset) {
                    callback(_result.slice(_offset));
                }

                if (_limit) {
                    callback(_result.slice(0, _limit));
                }

                if (!_result.length && _single) {
                    callback(null);
                }

                if (_single) {
                    callback(_result[0]);
                }

                if (_count) {
                    callback(_result.length);
                }

                callback(_result);
            }
        }
    }]);

    return EntitySubscriber;
}();